#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import datetime as dt
from unittest import mock
from unittest.mock import MagicMock, patch

import pytest
from openlineage.client.transport.composite import CompositeConfig, CompositeTransport
from openlineage.client.transport.http import HttpConfig, HttpTransport
from openlineage.client.transport.kafka import KafkaConfig, KafkaTransport

from airflow.providers.openlineage.utils.spark import (
    _get_parent_job_information_as_spark_properties,
    _get_transport_information_as_spark_properties,
    _is_parent_job_information_present_in_spark_properties,
    _is_transport_information_present_in_spark_properties,
    inject_parent_job_information_into_spark_properties,
    inject_transport_information_into_spark_properties,
)

EXAMPLE_CONTEXT = {
    "ti": MagicMock(
        dag_id="dag_id",
        task_id="task_id",
        try_number=1,
        map_index=1,
        logical_date=dt.datetime(2024, 11, 11),
        dag_run=MagicMock(logical_date=dt.datetime(2024, 11, 11), clear_number=0),
    )
}
EXAMPLE_HTTP_TRANSPORT_CONFIG = {
    "type": "http",
    "url": "https://some-custom.url",
    "endpoint": "/api/custom",
    "timeout": 123,
    "compression": "gzip",
    "custom_headers": {
        "key1": "val1",
        "key2": "val2",
    },
    "auth": {
        "type": "api_key",
        "apiKey": "secret_123",
    },
}
EXAMPLE_KAFKA_TRANSPORT_CONFIG = {
    "type": "kafka",
    "topic": "my_topic",
    "config": {
        "bootstrap.servers": "test-kafka-hm0fo:10011,another.host-uuj0l:10012",
        "acks": "all",
        "retries": "3",
    },
    "flush": True,
    "messageKey": "some",
}
EXAMPLE_PARENT_JOB_SPARK_PROPERTIES = {
    "spark.openlineage.parentJobName": "dag_id.task_id",
    "spark.openlineage.parentJobNamespace": "default",
    "spark.openlineage.parentRunId": "01931885-2800-7be7-aa8d-aaa15c337267",
    "spark.openlineage.rootParentRunId": "01931885-2800-799d-8041-88a263ffa0d8",
    "spark.openlineage.rootParentJobName": "dag_id",
    "spark.openlineage.rootParentJobNamespace": "default",
}
EXAMPLE_TRANSPORT_SPARK_PROPERTIES = {
    "spark.openlineage.transport.type": "http",
    "spark.openlineage.transport.url": "https://some-custom.url",
    "spark.openlineage.transport.endpoint": "/api/custom",
    "spark.openlineage.transport.auth.type": "api_key",
    "spark.openlineage.transport.auth.apiKey": "Bearer secret_123",
    "spark.openlineage.transport.compression": "gzip",
    "spark.openlineage.transport.headers.key1": "val1",
    "spark.openlineage.transport.headers.key2": "val2",
    "spark.openlineage.transport.timeoutInMillis": "123000",
}

EXAMPLE_COMPOSITE_TRANSPORT_SPARK_PROPERTIES = {
    "spark.openlineage.transport.type": "composite",
    "spark.openlineage.transport.continueOnFailure": "True",
    "spark.openlineage.transport.transports.http.type": "http",
    "spark.openlineage.transport.transports.http.url": "https://some-custom.url",
    "spark.openlineage.transport.transports.http.endpoint": "/api/custom",
    "spark.openlineage.transport.transports.http.auth.type": "api_key",
    "spark.openlineage.transport.transports.http.auth.apiKey": "Bearer secret_123",
    "spark.openlineage.transport.transports.http.compression": "gzip",
    "spark.openlineage.transport.transports.http.headers.key1": "val1",
    "spark.openlineage.transport.transports.http.headers.key2": "val2",
    "spark.openlineage.transport.transports.http.timeoutInMillis": "123000",
}


def test_get_parent_job_information_as_spark_properties():
    result = _get_parent_job_information_as_spark_properties(EXAMPLE_CONTEXT)
    assert result == EXAMPLE_PARENT_JOB_SPARK_PROPERTIES


@patch("airflow.providers.openlineage.utils.spark.get_openlineage_listener")
def test_get_transport_information_as_spark_properties(mock_ol_listener):
    fake_listener = mock.MagicMock()
    mock_ol_listener.return_value = fake_listener
    fake_listener.adapter.get_or_create_openlineage_client.return_value.transport = HttpTransport(
        HttpConfig.from_dict(EXAMPLE_HTTP_TRANSPORT_CONFIG)
    )
    result = _get_transport_information_as_spark_properties()
    assert result == EXAMPLE_TRANSPORT_SPARK_PROPERTIES


@patch("airflow.providers.openlineage.utils.spark.get_openlineage_listener")
def test_get_transport_information_as_spark_properties_unsupported_transport_type(mock_ol_listener):
    kafka_config = KafkaConfig(
        topic="my_topic",
        config={
            "bootstrap.servers": "test-kafka-hm0fo:10011,another.host-uuj0l:10012",
            "acks": "all",
            "retries": "3",
        },
        flush=True,
        messageKey="some",
    )
    fake_listener = mock.MagicMock()
    mock_ol_listener.return_value = fake_listener
    fake_listener.adapter.get_or_create_openlineage_client.return_value.transport = KafkaTransport(
        kafka_config
    )
    result = _get_transport_information_as_spark_properties()
    assert result == {}


@patch("airflow.providers.openlineage.utils.spark.get_openlineage_listener")
def test_get_transport_information_as_spark_properties_composite_transport_type(mock_ol_listener):
    fake_listener = mock.MagicMock()
    mock_ol_listener.return_value = fake_listener
    fake_listener.adapter.get_or_create_openlineage_client.return_value.transport = CompositeTransport(
        CompositeConfig.from_dict(
            {"transports": {"http": EXAMPLE_HTTP_TRANSPORT_CONFIG, "kafka": EXAMPLE_KAFKA_TRANSPORT_CONFIG}}
        )
    )
    result = _get_transport_information_as_spark_properties()
    assert result == EXAMPLE_COMPOSITE_TRANSPORT_SPARK_PROPERTIES


@pytest.mark.parametrize(
    ("properties", "expected"),
    [
        (
            {"spark.openlineage.parentJobNamespace": "example_namespace"},
            True,
        ),
        (
            {"spark.openlineage.parentJobName": "some_job_name"},
            True,
        ),
        (
            {"spark.openlineage.parentRunId": "some_run_id"},
            True,
        ),
        (
            {"spark.openlineage.parentWhatever": "some_value", "some.other.property": "value"},
            True,
        ),
        (
            {"some.other.property": "value"},
            False,
        ),
        (
            {
                "spark.openlineage.parentJobNamespace": "another_namespace",
                "spark.openlineage.parentJobName": "another_job_name",
                "spark.openlineage.parentRunId": "another_run_id",
                "spark.openlineage.rootParentJobNamespace": "another_namespace",
                "spark.openlineage.rootParentJobName": "another_job_name",
                "spark.openlineage.rootParentRunId": "another_run_id",
            },
            True,
        ),
        (
            {},
            False,
        ),
    ],
)
def test_is_parent_job_information_present_in_spark_properties(properties, expected):
    assert _is_parent_job_information_present_in_spark_properties(properties) is expected


@pytest.mark.parametrize(
    ("properties", "expected"),
    [
        (
            {"spark.openlineage.transport": "example_namespace"},
            True,
        ),
        (
            {"spark.openlineage.transport.type": "some_job_name"},
            True,
        ),
        (
            {"spark.openlineage.transport.urlParams.value1": "some_run_id"},
            True,
        ),
        (
            {"spark.openlineage.transportWhatever": "some_value", "some.other.property": "value"},
            True,
        ),
        (
            {"some.other.property": "value"},
            False,
        ),
        (
            {},
            False,
        ),
    ],
)
def test_is_transport_information_present_in_spark_properties(properties, expected):
    assert _is_transport_information_present_in_spark_properties(properties) is expected


@pytest.mark.parametrize(
    ("properties", "should_inject"),
    [
        (
            {"spark.openlineage.parentJobNamespace": "example_namespace"},
            False,
        ),
        (
            {"spark.openlineage.parentJobName": "some_job_name"},
            False,
        ),
        (
            {"spark.openlineage.parentRunId": "some_run_id"},
            False,
        ),
        (
            {"spark.openlineage.parentWhatever": "some_value", "some.other.property": "value"},
            False,
        ),
        (
            {"some.other.property": "value"},
            True,
        ),
        (
            {},
            True,
        ),
        (
            {
                "spark.openlineage.parentJobNamespace": "another_namespace",
                "spark.openlineage.parentJobName": "another_job_name",
                "spark.openlineage.parentRunId": "another_run_id",
            },
            False,
        ),
    ],
)
def test_inject_parent_job_information_into_spark_properties(properties, should_inject):
    result = inject_parent_job_information_into_spark_properties(properties, EXAMPLE_CONTEXT)
    expected = {**properties, **EXAMPLE_PARENT_JOB_SPARK_PROPERTIES} if should_inject else properties
    assert result == expected


@pytest.mark.parametrize(
    ("properties", "should_inject"),
    [
        (
            {"spark.openlineage.transport": "example_namespace"},
            False,
        ),
        (
            {"spark.openlineage.transport.type": "some_job_name"},
            False,
        ),
        (
            {"spark.openlineage.transport.url": "some_run_id"},
            False,
        ),
        (
            {"spark.openlineage.transportWhatever": "some_value", "some.other.property": "value"},
            False,
        ),
        (
            {"some.other.property": "value"},
            True,
        ),
        (
            {},
            True,
        ),
    ],
)
@patch("airflow.providers.openlineage.utils.spark.get_openlineage_listener")
def test_inject_transport_information_into_spark_properties(mock_ol_listener, properties, should_inject):
    fake_listener = mock.MagicMock()
    mock_ol_listener.return_value = fake_listener
    fake_listener.adapter.get_or_create_openlineage_client.return_value.transport = HttpTransport(
        HttpConfig.from_dict(EXAMPLE_HTTP_TRANSPORT_CONFIG)
    )
    result = inject_transport_information_into_spark_properties(properties, EXAMPLE_CONTEXT)
    expected = {**properties, **EXAMPLE_TRANSPORT_SPARK_PROPERTIES} if should_inject else properties
    assert result == expected


@pytest.mark.parametrize(
    ("properties", "should_inject"),
    [
        (
            {"spark.openlineage.transport": "example_namespace"},
            False,
        ),
        (
            {"spark.openlineage.transport.type": "some_job_name"},
            False,
        ),
        (
            {"spark.openlineage.transport.url": "some_run_id"},
            False,
        ),
        (
            {"spark.openlineage.transportWhatever": "some_value", "some.other.property": "value"},
            False,
        ),
        (
            {"some.other.property": "value"},
            True,
        ),
        (
            {},
            True,
        ),
    ],
)
@patch("airflow.providers.openlineage.utils.spark.get_openlineage_listener")
def test_inject_composite_transport_information_into_spark_properties(
    mock_ol_listener, properties, should_inject
):
    fake_listener = mock.MagicMock()
    mock_ol_listener.return_value = fake_listener
    fake_listener.adapter.get_or_create_openlineage_client.return_value.transport = CompositeTransport(
        CompositeConfig(
            transports={
                "http": EXAMPLE_HTTP_TRANSPORT_CONFIG,
                "console": {"type": "console"},
            },
            continue_on_failure=True,
        )
    )
    result = inject_transport_information_into_spark_properties(properties, EXAMPLE_CONTEXT)
    expected = {**properties, **EXAMPLE_COMPOSITE_TRANSPORT_SPARK_PROPERTIES} if should_inject else properties
    assert result == expected



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Simple DAG without schedule and extra args, with one operator, to verify OpenLineage event integrity.

It checks:
    - required keys
    - field formats and types
    - number of task events (one start, one complete)
"""

from __future__ import annotations

from datetime import datetime

from airflow import DAG
from airflow.providers.standard.operators.python import PythonOperator

from system.openlineage.expected_events import get_expected_event_file_path
from system.openlineage.operator import OpenLineageTestOperator

try:
    from airflow.sdk import chain
except ImportError:
    from airflow.models.baseoperator import chain  # type: ignore[no-redef]


def do_nothing():
    pass


DAG_ID = "openlineage_base_simple_dag"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime(2021, 1, 1),
    schedule=None,
    catchup=False,
    default_args={"retries": 0},
) as dag:
    do_nothing_task = PythonOperator(task_id="do_nothing_task", python_callable=do_nothing)

    check_events = OpenLineageTestOperator(
        task_id="check_events", file_path=get_expected_event_file_path(DAG_ID)
    )

    chain(do_nothing_task, check_events)


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from openlineage.client.serde import Serde
from openlineage.client.transport import Config, Transport, get_default_factory

# We use `airflow.models.variable.Variable` instead of `airflow.sdk.Variable`, even for Airflow 3, as it also
# works on scheduler. It may emit DeprecationWarning on workers, but it's needed for DAG events to be emitted.
from airflow.models.variable import Variable

if TYPE_CHECKING:
    from openlineage.client.client import Event

log = logging.getLogger(__name__)


class VariableTransport(Transport):
    """
    This transport sends OpenLineage events to Variables.

    Key schema is <DAG_ID>.<TASK_ID>.event.<EVENT_TYPE>.
    It's made to be used in system tests, stored data read by OpenLineageTestOperator.
    """

    kind = "variable"

    def __init__(self, config: Config) -> None:
        log.debug("Constructing OpenLineage transport that will send events to Airflow Variables.")

    def emit(self, event: Event) -> None:
        key = f"{event.job.name}.event.{event.eventType.value.lower()}"  # type: ignore[union-attr]
        event_str = Serde.to_json(event)
        if (var := Variable.get(key=key, default_var=None, deserialize_json=True)) is not None:
            log.debug("Appending OL event to Airflow Variable `%s`", key)
            Variable.set(key=key, value=var + [event_str], serialize_json=True)
        else:
            log.debug("Emitting OL event to new Airflow Variable `%s`", key)
            Variable.set(key=key, value=[event_str], serialize_json=True)


get_default_factory().register_transport(VariableTransport.kind, VariableTransport)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import os
from unittest import mock

import pytest

from airflow.exceptions import AirflowConfigException
from airflow.providers.openlineage.conf import (
    _is_true,
    config_path,
    custom_extractors,
    custom_run_facets,
    dag_state_change_process_pool_size,
    debug_mode,
    disabled_operators,
    execution_timeout,
    include_full_task_info,
    is_disabled,
    is_source_enabled,
    namespace,
    selective_enable,
    spark_inject_parent_job_info,
    spark_inject_transport_info,
    transport,
)

from tests_common.test_utils.config import conf_vars, env_vars

_CONFIG_SECTION = "openlineage"
_VAR_CONFIG_PATH = "OPENLINEAGE_CONFIG"
_CONFIG_OPTION_CONFIG_PATH = "config_path"
_CONFIG_OPTION_CUSTOM_RUN_FACETS = "custom_run_facets"
_VAR_DISABLE_SOURCE_CODE = "OPENLINEAGE_AIRFLOW_DISABLE_SOURCE_CODE"
_CONFIG_OPTION_DISABLE_SOURCE_CODE = "disable_source_code"
_CONFIG_OPTION_DISABLED_FOR_OPERATORS = "disabled_for_operators"
_CONFIG_OPTION_EXECUTION_TIMEOUT = "execution_timeout"
_VAR_EXTRACTORS = "OPENLINEAGE_EXTRACTORS"
_CONFIG_OPTION_EXTRACTORS = "extractors"
_VAR_NAMESPACE = "OPENLINEAGE_NAMESPACE"
_CONFIG_OPTION_NAMESPACE = "namespace"
_CONFIG_OPTION_TRANSPORT = "transport"
_VAR_DISABLED = "OPENLINEAGE_DISABLED"
_CONFIG_OPTION_DISABLED = "disabled"
_VAR_URL = "OPENLINEAGE_URL"
_CONFIG_OPTION_SELECTIVE_ENABLE = "selective_enable"
_CONFIG_OPTION_DAG_STATE_CHANGE_PROCESS_POOL_SIZE = "dag_state_change_process_pool_size"
_CONFIG_OPTION_INCLUDE_FULL_TASK_INFO = "include_full_task_info"
_CONFIG_OPTION_DEBUG_MODE = "debug_mode"
_CONFIG_OPTION_SPARK_INJECT_PARENT_JOB_INFO = "spark_inject_parent_job_info"
_CONFIG_OPTION_SPARK_INJECT_TRANSPORT_INFO = "spark_inject_transport_info"

_BOOL_PARAMS = (
    ("1", True),
    ("t", True),
    ("T", True),
    ("tRuE ", True),
    (" true", True),
    ("TRUE", True),
    ("0", False),
    ("f", False),
    ("F", False),
    (" fAlSe", False),
    ("false ", False),
    ("FALSE", False),
)


@pytest.mark.parametrize(
    ("var_string", "expected"),
    (
        *_BOOL_PARAMS,
        ("some_string", False),
        ("aasd123", False),
        (True, True),
        (False, False),
    ),
)
def test_is_true(var_string, expected):
    assert _is_true(var_string) is expected


@env_vars({_VAR_CONFIG_PATH: "env_var_path"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): None})
def test_config_path_legacy_env_var_is_used_when_no_conf_option_set():
    assert config_path() == "env_var_path"


@env_vars({_VAR_CONFIG_PATH: "env_var_path"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "config_path"})
def test_config_path_conf_option_has_precedence_over_legacy_env_var():
    assert config_path() == "config_path"


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): ""})
def test_config_path_empty_conf_option():
    assert config_path() == ""


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): None})
def test_config_path_do_not_fail_if_conf_option_missing():
    assert config_path() == ""


@pytest.mark.parametrize(
    ("var_string", "expected"),
    _BOOL_PARAMS,
)
def test_disable_source_code(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): var_string}):
        result = is_source_enabled()
        assert result is not expected  # conf is disabled_... and func is enabled_... hence the `not` here


@env_vars({_VAR_DISABLE_SOURCE_CODE: "true"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): None})
def test_disable_source_code_legacy_env_var_is_used_when_no_conf_option_set():
    assert is_source_enabled() is False


@env_vars({_VAR_DISABLE_SOURCE_CODE: "false"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): "true"})
def test_disable_source_code_conf_option_has_precedence_over_legacy_env_var():
    assert is_source_enabled() is False


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): "asdadawlaksnd"})
def test_disable_source_code_conf_option_not_working_for_random_string():
    with pytest.raises(AirflowConfigException):
        is_source_enabled()


@env_vars({_VAR_DISABLE_SOURCE_CODE: "asdadawlaksnd"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): None})
def test_disable_source_code_legacy_env_var_not_working_for_random_string():
    assert is_source_enabled() is True


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): ""})
def test_disable_source_code_empty_conf_option():
    with pytest.raises(AirflowConfigException):
        is_source_enabled()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLE_SOURCE_CODE): None})
def test_disable_source_code_do_not_fail_if_conf_option_missing():
    assert is_source_enabled() is True


@pytest.mark.parametrize(
    ("var_string", "expected"),
    _BOOL_PARAMS,
)
def test_selective_enable(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SELECTIVE_ENABLE): var_string}):
        result = selective_enable()
        assert result is expected


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SELECTIVE_ENABLE): "asdadawlaksnd"})
def test_selective_enable_not_working_for_random_string():
    with pytest.raises(AirflowConfigException):
        selective_enable()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SELECTIVE_ENABLE): ""})
def test_selective_enable_empty_conf_option():
    with pytest.raises(AirflowConfigException):
        selective_enable()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SELECTIVE_ENABLE): None})
def test_selective_enable_do_not_fail_if_conf_option_missing():
    assert selective_enable() is False


@pytest.mark.parametrize(
    ("var_string", "expected"),
    (
        ("    ", {}),
        ("  ;   ", {}),
        (";", {}),
        ("path.to.Operator  ;", {"path.to.Operator"}),
        ("  ;  path.to.Operator  ;", {"path.to.Operator"}),
        ("path.to.Operator", {"path.to.Operator"}),
        ("path.to.Operator  ;   path.to.second.Operator ; ", {"path.to.Operator", "path.to.second.Operator"}),
        ("path.to.Operator;path.to.second.Operator", {"path.to.Operator", "path.to.second.Operator"}),
    ),
)
def test_disabled_for_operators(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLED_FOR_OPERATORS): var_string}):
        result = disabled_operators()
        assert isinstance(result, set)
        assert sorted(result) == sorted(expected)


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLED_FOR_OPERATORS): ""})
def test_disabled_for_operators_empty_conf_option():
    assert disabled_operators() == set()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLED_FOR_OPERATORS): None})
def test_disabled_for_operators_do_not_fail_if_conf_option_missing():
    assert disabled_operators() == set()


@env_vars({_VAR_EXTRACTORS: "path.Extractor"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXTRACTORS): None})
def test_extractors_legacy_legacy_env_var_is_used_when_no_conf_option_set():
    assert os.getenv(_VAR_EXTRACTORS) == "path.Extractor"
    assert custom_extractors() == {"path.Extractor"}


@env_vars({_VAR_EXTRACTORS: "env.Extractor"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXTRACTORS): "conf.Extractor"})
def test_extractors_conf_option_has_precedence_over_legacy_env_var():
    assert os.getenv(_VAR_EXTRACTORS) == "env.Extractor"
    assert custom_extractors() == {"conf.Extractor"}


@pytest.mark.parametrize(
    ("var_string", "expected"),
    (
        ("path.to.Extractor  ;", {"path.to.Extractor"}),
        ("  ;  path.to.Extractor  ;", {"path.to.Extractor"}),
        ("path.to.Extractor", {"path.to.Extractor"}),
        (
            "path.to.Extractor  ;   path.to.second.Extractor ; ",
            {"path.to.Extractor", "path.to.second.Extractor"},
        ),
        ("path.to.Extractor;path.to.second.Extractor", {"path.to.Extractor", "path.to.second.Extractor"}),
    ),
)
def test_extractors(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXTRACTORS): var_string}):
        result = custom_extractors()
        assert isinstance(result, set)
        assert sorted(result) == sorted(expected)


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXTRACTORS): ""})
def test_extractors_empty_conf_option():
    assert custom_extractors() == set()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXTRACTORS): None})
def test_extractors_do_not_fail_if_conf_option_missing():
    assert custom_extractors() == set()


@conf_vars(dict())
def test_custom_run_facets_not_set():
    assert custom_run_facets() == set()


def test_custom_run_facets_with_no_values():
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_CUSTOM_RUN_FACETS): None}):
        assert custom_run_facets() == set()
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_CUSTOM_RUN_FACETS): ""}):
        assert custom_run_facets() == set()


@conf_vars(
    {
        (
            _CONFIG_SECTION,
            _CONFIG_OPTION_CUSTOM_RUN_FACETS,
        ): " unit.my_function;; unit.my_function ; my_function_2; ",
    }
)
def test_custom_run_facets():
    assert custom_run_facets() == {"unit.my_function", "my_function_2"}


@env_vars({_VAR_NAMESPACE: "my_custom_namespace"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_NAMESPACE): None})
def test_namespace_legacy_env_var_is_used_when_no_conf_option_set():
    assert os.getenv(_VAR_NAMESPACE) == "my_custom_namespace"
    assert namespace() == "my_custom_namespace"


@env_vars({_VAR_NAMESPACE: "env_namespace"})
@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_NAMESPACE): "my_custom_namespace"})
def test_namespace_conf_option_has_precedence_over_legacy_env_var():
    assert os.getenv(_VAR_NAMESPACE) == "env_namespace"
    assert namespace() == "my_custom_namespace"


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_NAMESPACE): ""})
def test_namespace_empty_conf_option():
    assert namespace() == "default"


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_NAMESPACE): None})
def test_namespace_do_not_fail_if_conf_option_missing():
    assert namespace() == "default"


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): '{"valid": "json"}'})
def test_transport_valid():
    assert transport() == {"valid": "json"}


@pytest.mark.parametrize("transport_value", ('["a", "b"]', "[]", '[{"a": "b"}]'))
def test_transport_not_valid(transport_value):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): transport_value}):
        with pytest.raises(ValueError, match="OpenLineage transport"):
            transport()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): ""})
def test_transport_empty_conf_option():
    assert transport() == {}


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): None})
def test_transport_do_not_fail_if_conf_option_missing():
    assert transport() == {}


@pytest.mark.parametrize("disabled", ("1", "t", "T", "true", "TRUE", "True"))
@mock.patch.dict(os.environ, {_VAR_URL: ""}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_possible_values_for_disabling(disabled):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): disabled}):
        assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "https://test.com"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): "asdadawlaksnd",
    }
)
def test_is_disabled_raises_for_random_string():
    with pytest.raises(AirflowConfigException):
        is_disabled()


@mock.patch.dict(os.environ, {_VAR_URL: "https://test.com"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): "",
    }
)
def test_is_disabled_raises_error_for_empty_string():
    with pytest.raises(AirflowConfigException):
        is_disabled()


@mock.patch.dict(os.environ, {_VAR_URL: "https://test.com"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_is_false_when_not_explicitly_disabled_and_url_set():
    assert is_disabled() is False


@mock.patch.dict(os.environ, {_VAR_URL: ""}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): '{"valid": "transport"}',
    }
)
def test_is_disabled_is_false_when_not_explicitly_disabled_and_transport_set():
    assert is_disabled() is False


@mock.patch.dict(os.environ, {_VAR_URL: ""}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "some/path.yml",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_is_false_when_not_explicitly_disabled_and_config_path_set():
    assert is_disabled() is False


@mock.patch.dict(os.environ, {_VAR_URL: "https://test.com"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "some/path.yml",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): '{"valid": "transport"}',
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): "true",
    }
)
def test_is_disabled_conf_option_is_enough_to_disable():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "https://test.com", _VAR_DISABLED: "true"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "some/path.yml",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): '{"valid": "transport"}',
    }
)
def test_is_disabled_legacy_env_var_is_enough_to_disable():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "", _VAR_DISABLED: "true"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): None,
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): None,
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): None,
    }
)
def test_is_disabled_legacy_env_var_is_used_when_no_config():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "", _VAR_DISABLED: "false"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "some/path.yml",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): "true",
    }
)
def test_is_disabled_conf_true_has_precedence_over_env_var_false():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "", _VAR_DISABLED: "true"}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "some/path.yml",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): "false",
    }
)
def test_is_disabled_env_var_true_has_precedence_over_conf_false():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: ""}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_empty_conf_option():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "", "OPENLINEAGE__TRANSPORT": '{"type": "console"}'}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_env_variables_present():
    assert is_disabled() is False


@mock.patch.dict(os.environ, {_VAR_URL: "", "OPENLINEAGE__TRANSPORT": ""}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_env_variables_not_present():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: "", "OPENLINEAGE__TRANSPOR": '{"type": "console"}'}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
    }
)
def test_is_disabled_env_variables_not_present_typo():
    assert is_disabled() is True


@mock.patch.dict(os.environ, {_VAR_URL: ""}, clear=True)
@conf_vars(
    {
        (_CONFIG_SECTION, _CONFIG_OPTION_CONFIG_PATH): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_TRANSPORT): "",
        (_CONFIG_SECTION, _CONFIG_OPTION_DISABLED): None,
    }
)
def test_is_disabled_do_not_fail_if_conf_option_missing():
    assert is_disabled() is True


@pytest.mark.parametrize(
    ("var_string", "expected"),
    (
        ("1", 1),
        ("2   ", 2),
        ("  3", 3),
        (None, 1),  # default
    ),
)
def test_dag_state_change_process_pool_size(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DAG_STATE_CHANGE_PROCESS_POOL_SIZE): var_string}):
        result = dag_state_change_process_pool_size()
        assert result == expected


@pytest.mark.parametrize(
    "var_string",
    (
        "4.56",
        "asdf",
        "true",
        "false",
        "None",
        "",
        " ",
    ),
)
def test_dag_state_change_process_pool_size_invalid_value_raise_error(var_string):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DAG_STATE_CHANGE_PROCESS_POOL_SIZE): var_string}):
        with pytest.raises(AirflowConfigException):
            dag_state_change_process_pool_size()


@pytest.mark.parametrize(
    ("var_string", "expected"),
    (
        ("1", 1),
        ("2   ", 2),
        ("  3", 3),
        (None, 10),  # default
    ),
)
def test_execution_timeout(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXECUTION_TIMEOUT): var_string}):
        result = execution_timeout()
        assert result == expected


@pytest.mark.parametrize(
    "var_string",
    (
        "4.56",
        "asdf",
        "true",
        "false",
        "None",
        "",
        " ",
    ),
)
def test_execution_timeout_invalid_value_raise_error(var_string):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_EXECUTION_TIMEOUT): var_string}):
        with pytest.raises(AirflowConfigException):
            execution_timeout()


@pytest.mark.parametrize(
    ("var_string", "expected"),
    _BOOL_PARAMS,
)
def test_include_full_task_info(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_INCLUDE_FULL_TASK_INFO): var_string}):
        result = include_full_task_info()
        assert result is expected


@pytest.mark.parametrize(
    "var_string",
    (
        "a",
        "asdf",
        "None",
        "31",
        "",
        " ",
    ),
)
def test_include_full_task_info_invalid_value_raise_error(var_string):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_INCLUDE_FULL_TASK_INFO): var_string}):
        with pytest.raises(AirflowConfigException):
            include_full_task_info()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_INCLUDE_FULL_TASK_INFO): None})
def test_include_full_task_info_do_not_fail_if_conf_option_missing():
    assert include_full_task_info() is False


@pytest.mark.parametrize(
    ("var_string", "expected"),
    _BOOL_PARAMS,
)
def test_debug_mode(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DEBUG_MODE): var_string}):
        result = debug_mode()
        assert result is expected


@pytest.mark.parametrize(
    "var_string",
    (
        "a",
        "asdf",
        "None",
        "31",
        "",
        " ",
    ),
)
def test_debug_mode_invalid_value_raise_error(var_string):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DEBUG_MODE): var_string}):
        with pytest.raises(AirflowConfigException):
            debug_mode()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_DEBUG_MODE): None})
def test_debug_mode_do_not_fail_if_conf_option_missing():
    assert debug_mode() is False


@pytest.mark.parametrize(
    ("var_string", "expected"),
    _BOOL_PARAMS,
)
def test_spark_inject_parent_job_info(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_PARENT_JOB_INFO): var_string}):
        result = spark_inject_parent_job_info()
        assert result is expected


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_PARENT_JOB_INFO): "asdadawlaksnd"})
def test_spark_inject_parent_job_info_not_working_for_random_string():
    with pytest.raises(AirflowConfigException):
        spark_inject_parent_job_info()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_PARENT_JOB_INFO): ""})
def test_spark_inject_parent_job_info_empty_conf_option():
    with pytest.raises(AirflowConfigException):
        spark_inject_parent_job_info()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_PARENT_JOB_INFO): None})
def test_spark_inject_parent_job_info_do_not_fail_if_conf_option_missing():
    assert spark_inject_parent_job_info() is False


@pytest.mark.parametrize(
    ("var_string", "expected"),
    _BOOL_PARAMS,
)
def test_spark_inject_transport_info(var_string, expected):
    with conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_TRANSPORT_INFO): var_string}):
        result = spark_inject_transport_info()
        assert result is expected


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_TRANSPORT_INFO): "asdadawlaksnd"})
def test_spark_inject_transport_info_not_working_for_random_string():
    with pytest.raises(AirflowConfigException):
        spark_inject_transport_info()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_TRANSPORT_INFO): ""})
def test_spark_inject_transport_info_empty_conf_option():
    with pytest.raises(AirflowConfigException):
        spark_inject_transport_info()


@conf_vars({(_CONFIG_SECTION, _CONFIG_OPTION_SPARK_INJECT_TRANSPORT_INFO): None})
def test_spark_inject_transport_info_do_not_fail_if_conf_option_missing():
    assert spark_inject_transport_info() is False



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import pickle

import attrs
import pytest

from airflow.providers.openlineage.plugins.facets import (
    AirflowDagRunFacet,
    AirflowJobFacet,
    AirflowRunFacet,
    AirflowStateRunFacet,
)


def test_airflow_run_facet():
    dag = {"dag_id": "123"}
    dag_run = {"dag_run_id": "456"}
    task = {"task_id": "789"}
    task_instance = {"task_instance_id": "000"}
    task_uuid = "XXX"

    airflow_run_facet = AirflowRunFacet(
        dag=dag,
        dagRun=dag_run,
        task=task,
        taskInstance=task_instance,
        taskUuid=task_uuid,
    )

    assert airflow_run_facet.dag == dag
    assert airflow_run_facet.dagRun == dag_run
    assert airflow_run_facet.task == task
    assert airflow_run_facet.taskInstance == task_instance
    assert airflow_run_facet.taskUuid == task_uuid


def test_airflow_dag_run_facet():
    dag = {"dag_id": "123"}
    dag_run = {"dag_run_id": "456"}

    airflow_dag_run_facet = AirflowDagRunFacet(
        dag=dag,
        dagRun=dag_run,
    )

    assert airflow_dag_run_facet.dag == dag
    assert airflow_dag_run_facet.dagRun == dag_run


@pytest.mark.parametrize(
    "instance",
    [
        pytest.param(
            AirflowJobFacet(
                taskTree={"task_0": {"section_1.task_3": {}}},
                taskGroups={
                    "section_1": {
                        "parent_group": None,
                        "tooltip": "",
                        "ui_color": "CornflowerBlue",
                        "ui_fgcolor": "#000",
                        "ui_label": "section_1",
                    }
                },
                tasks={
                    "task_0": {
                        "operator": "airflow.providers.standard.operators.bash.BashOperator",
                        "task_group": None,
                        "emits_ol_events": True,
                        "ui_color": "#f0ede4",
                        "ui_fgcolor": "#000",
                        "ui_label": "task_0",
                        "is_setup": False,
                        "is_teardown": False,
                    }
                },
            ),
            id="AirflowJobFacet",
        ),
        pytest.param(
            AirflowStateRunFacet(
                dagRunState="SUCCESS",
                tasksState={"task_0": "SKIPPED"},
                tasksDuration={"task_0": 123},
            ),
            id="AirflowStateRunFacet",
        ),
        pytest.param(
            AirflowDagRunFacet(
                dag={
                    "timetable": {"delta": 86400.0},
                    "owner": "airflow",
                    "start_date": "2024-06-01T00:00:00+00:00",
                },
                dagRun={"conf": {}, "dag_id": "dag_id"},
            ),
            id="AirflowDagRunFacet",
        ),
    ],
)
def test_facets_are_pickled_correctly(instance):
    cls = instance.__class__
    instance = pickle.loads(pickle.dumps(instance))
    for field in attrs.fields(cls):
        getattr(instance, field.name)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Simple DAG that triggers another simple DAG.

It checks:
    - task's trigger_dag_id
    - DAGRun START and COMPLETE events, for the triggered DAG
    - propagation of OL parent and root info from DAGRun conf
"""

from __future__ import annotations

from datetime import datetime

from airflow import DAG
from airflow.providers.standard.operators.bash import BashOperator
from airflow.providers.standard.operators.trigger_dagrun import TriggerDagRunOperator

from system.openlineage.expected_events import get_expected_event_file_path
from system.openlineage.operator import OpenLineageTestOperator

DAG_ID = "openlineage_trigger_dag"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime(2021, 1, 1),
    schedule=None,
    catchup=False,
    default_args={"retries": 0},
) as dag:
    trigger_dagrun = TriggerDagRunOperator(
        task_id="trigger_dagrun",
        trigger_dag_id="openlineage_trigger_dag_child__notrigger",
        trigger_run_id=f"openlineage_trigger_dag_triggering_child_{datetime.now().isoformat()}",
        wait_for_completion=True,
        conf={
            "some_config": "value1",
            "openlineage": {
                "parentRunId": "3bb703d1-09c1-4a42-8da5-35a0b3216072",
                "parentJobNamespace": "prod_biz",
                "parentJobName": "get_files",
                "rootParentRunId": "9d3b14f7-de91-40b6-aeef-e887e2c7673e",
                "rootParentJobNamespace": "prod_analytics",
                "rootParentJobName": "generate_report_sales_e2e",
            },
        },
        poke_interval=10,
    )

    check_events = OpenLineageTestOperator(
        task_id="check_events", file_path=get_expected_event_file_path(DAG_ID)
    )

    trigger_dagrun >> check_events


with DAG(
    dag_id="openlineage_trigger_dag_child__notrigger",
    start_date=datetime(2021, 1, 1),
    schedule=None,
    catchup=False,
    tags=["first", "second@", "with'quote", 'z"e'],
    doc_md="MD DAG doc",
    description="DAG description",
    default_args={"retries": 0},
) as child_dag:
    do_nothing_task = BashOperator(task_id="do_nothing_task", bash_command="sleep 10;")


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import datetime
import json
import uuid
from json import JSONEncoder
from typing import Any
from unittest.mock import MagicMock, patch

import pytest
from attrs import define
from openlineage.client.utils import RedactMixin
from pkg_resources import parse_version

from airflow.providers.common.compat.assets import Asset
from airflow.providers.common.compat.sdk import timezone
from airflow.providers.openlineage.plugins.facets import AirflowDebugRunFacet
from airflow.providers.openlineage.utils.utils import (
    DagInfo,
    InfoJsonEncodable,
    OpenLineageRedactor,
    _get_all_packages_installed,
    _is_name_redactable,
    get_airflow_debug_facet,
    get_airflow_run_facet,
    get_fully_qualified_class_name,
    get_processing_engine_facet,
    is_operator_disabled,
)
from airflow.serialization.enums import DagAttributeTypes, Encoding
from airflow.utils.state import State
from airflow.utils.types import DagRunType

from tests_common.test_utils.compat import (
    BashOperator,
)
from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS, AIRFLOW_V_3_1_PLUS

if AIRFLOW_V_3_1_PLUS:
    from airflow.models.dag import get_next_data_interval

if AIRFLOW_V_3_1_PLUS:
    from airflow.sdk._shared.secrets_masker import DEFAULT_SENSITIVE_FIELDS, SecretsMasker
elif AIRFLOW_V_3_0_PLUS:
    from airflow.sdk.execution_time.secrets_masker import (  # type: ignore[no-redef]
        DEFAULT_SENSITIVE_FIELDS,
        SecretsMasker,
    )
else:
    from airflow.utils.log.secrets_masker import (  # type: ignore[attr-defined,no-redef]
        DEFAULT_SENSITIVE_FIELDS,
        SecretsMasker,
    )

from airflow.providers.common.compat.sdk import DAG

if AIRFLOW_V_3_0_PLUS:
    from airflow.utils.types import DagRunTriggeredByType


class SafeStrDict(dict):
    def __str__(self):
        castable = []
        for key, val in self.items():
            try:
                str(key), str(val)
                castable.append((key, val))
            except (TypeError, NotImplementedError):
                continue
        return str(dict(castable))


@patch("airflow.providers.openlineage.utils.utils.metadata.distributions")
def test_get_all_packages_installed(mock_distributions):
    mock_distributions.return_value = [MagicMock(metadata={"Name": "package1"}, version="1.0.0")]
    assert _get_all_packages_installed() == {"package1": "1.0.0"}


@patch("airflow.providers.openlineage.utils.utils.conf.debug_mode", return_value=False)
def test_get_airflow_debug_facet_not_in_debug_mode(mock_debug_mode):
    assert get_airflow_debug_facet() == {}


@patch("airflow.providers.openlineage.utils.utils._get_all_packages_installed")
@patch("airflow.providers.openlineage.utils.utils.conf.debug_mode")
def test_get_airflow_debug_facet_logging_set_to_debug(mock_debug_mode, mock_get_packages):
    mock_debug_mode.return_value = True
    mock_get_packages.return_value = {"package1": "1.0.0"}
    result = get_airflow_debug_facet()
    expected_result = {"debug": AirflowDebugRunFacet(packages={"package1": "1.0.0"})}
    assert result == expected_result


@pytest.mark.db_test
@pytest.mark.need_serialized_dag
def test_get_dagrun_start_end(dag_maker):
    start_date = datetime.datetime(2022, 1, 1)
    end_date = datetime.datetime(2022, 1, 1, hour=2)
    with dag_maker("test", start_date=start_date, end_date=end_date, schedule="@once") as dag:
        pass
    dag_maker.sync_dagbag_to_db()

    run_id = str(uuid.uuid1())
    if AIRFLOW_V_3_1_PLUS:
        data_interval = get_next_data_interval(dag.timetable, dag_maker.dag_model)
    else:
        data_interval = dag.get_next_data_interval(dag_maker.dag_model)
    if AIRFLOW_V_3_0_PLUS:
        dagrun_kwargs = {
            "logical_date": data_interval.start,
            "run_after": data_interval.end,
            "triggered_by": DagRunTriggeredByType.TEST,
        }
    else:
        dagrun_kwargs = {"execution_date": data_interval.start}
    dagrun = dag.create_dagrun(
        state=State.NONE,
        run_id=run_id,
        run_type=DagRunType.MANUAL,
        data_interval=data_interval,
        **dagrun_kwargs,
    )
    assert dagrun.data_interval_start is not None
    start_date_tz = datetime.datetime(2022, 1, 1, tzinfo=timezone.utc)
    end_date_tz = datetime.datetime(2022, 1, 1, hour=2, tzinfo=timezone.utc)
    assert dagrun.data_interval_start, dagrun.data_interval_end == (start_date_tz, end_date_tz)


def test_parse_version():
    assert parse_version("2.3.0") >= parse_version("2.3.0.dev0")
    assert parse_version("2.3.0.dev0") >= parse_version("2.3.0.dev0")
    assert parse_version("2.3.0.beta1") >= parse_version("2.3.0.dev0")
    assert parse_version("2.3.1") >= parse_version("2.3.0.dev0")
    assert parse_version("2.4.0") >= parse_version("2.3.0.dev0")
    assert parse_version("3.0.0") >= parse_version("2.3.0.dev0")
    assert parse_version("2.2.0") < parse_version("2.3.0.dev0")
    assert parse_version("2.1.3") < parse_version("2.3.0.dev0")
    assert parse_version("2.2.4") < parse_version("2.3.0.dev0")
    assert parse_version("1.10.15") < parse_version("2.3.0.dev0")
    assert parse_version("2.2.4.dev0") < parse_version("2.3.0.dev0")


def test_safe_dict():
    assert str(SafeStrDict({"a": 1})) == str({"a": 1})

    class NotImplemented:
        def __str__(self):
            raise NotImplementedError

    assert str(SafeStrDict({"a": NotImplemented()})) == str({})


def test_info_json_encodable():
    class TestInfo(InfoJsonEncodable):
        excludes = ["exclude_1", "exclude_2", "imastring"]
        casts = {"iwanttobeint": lambda x: int(x.imastring)}
        renames = {"_faulty_name": "goody_name"}

    @define
    class Test:
        exclude_1: str
        imastring: str
        _faulty_name: str
        donotcare: str

    obj = Test("val", "123", "not_funny", "abc")

    assert json.loads(json.dumps(TestInfo(obj))) == {
        "iwanttobeint": 123,
        "goody_name": "not_funny",
        "donotcare": "abc",
    }


def test_info_json_encodable_without_slots():
    class TestInfo(InfoJsonEncodable):
        excludes = ["exclude_1", "exclude_2", "imastring"]
        casts = {"iwanttobeint": lambda x: int(x.imastring)}
        renames = {"_faulty_name": "goody_name"}

    @define(slots=False)
    class Test:
        exclude_1: str
        imastring: str
        _faulty_name: str
        donotcare: str

    obj = Test("val", "123", "not_funny", "abc")

    assert json.loads(json.dumps(TestInfo(obj))) == {
        "iwanttobeint": 123,
        "goody_name": "not_funny",
        "donotcare": "abc",
    }


def test_info_json_encodable_list_does_flatten():
    class TestInfo(InfoJsonEncodable):
        includes = ["alist"]

    @define
    class Test:
        alist: list[str]

    obj = Test(["a", "b", "c"])

    assert json.loads(json.dumps(TestInfo(obj))) == {"alist": "['a', 'b', 'c']"}


def test_info_json_encodable_list_does_include_nonexisting():
    class TestInfo(InfoJsonEncodable):
        includes = ["exists", "doesnotexist"]

    @define
    class Test:
        exists: str

    obj = Test("something")

    assert json.loads(json.dumps(TestInfo(obj))) == {"exists": "something"}


def test_is_name_redactable():
    class NotMixin:
        def __init__(self):
            self.password = "passwd"

    class Mixined(RedactMixin):
        _skip_redact = ["password"]

        def __init__(self):
            self.password = "passwd"
            self.transparent = "123"

    assert _is_name_redactable("password", NotMixin())
    assert not _is_name_redactable("password", Mixined())
    assert _is_name_redactable("transparent", Mixined())


@pytest.mark.enable_redact
def test_redact_with_exclusions(monkeypatch):
    sm = SecretsMasker()
    if AIRFLOW_V_3_1_PLUS:
        sm.sensitive_variables_fields = list(DEFAULT_SENSITIVE_FIELDS)
    redactor = OpenLineageRedactor.from_masker(sm)

    class NotMixin:
        def __init__(self):
            self.password = "passwd"

    class Proxy:
        pass

    def default(self, o):
        if isinstance(o, NotMixin):
            return o.__dict__
        raise TypeError

    assert redactor.redact(NotMixin()).password == "passwd"
    monkeypatch.setattr(JSONEncoder, "default", default)
    assert redactor.redact(NotMixin()).password == "***"

    assert redactor.redact(Proxy()) == "<<non-redactable: Proxy>>"
    assert redactor.redact({"a": "a", "b": Proxy()}) == {"a": "a", "b": "<<non-redactable: Proxy>>"}

    class Mixined(RedactMixin):
        _skip_redact = ["password"]

        def __init__(self):
            self.password = "passwd"
            self.transparent = "123"

    @define
    class NestedMixined(RedactMixin):
        _skip_redact = ["nested_field"]
        password: str
        nested_field: Any

    assert redactor.redact(Mixined()).password == "passwd"
    assert redactor.redact(Mixined()).transparent == "123"
    assert redactor.redact({"password": "passwd"}) == {"password": "***"}
    redacted_nested = redactor.redact(NestedMixined("passwd", NestedMixined("passwd", None)))
    assert redacted_nested == NestedMixined("***", NestedMixined("passwd", None))


def test_get_fully_qualified_class_name():
    from airflow.providers.openlineage.plugins.adapter import OpenLineageAdapter

    result = get_fully_qualified_class_name(BashOperator(task_id="test", bash_command="exit 0;"))
    assert result == "airflow.providers.standard.operators.bash.BashOperator"

    result = get_fully_qualified_class_name(OpenLineageAdapter())
    assert result == "airflow.providers.openlineage.plugins.adapter.OpenLineageAdapter"


@patch("airflow.providers.openlineage.conf.disabled_operators")
def test_is_operator_disabled(mock_disabled_operators):
    mock_disabled_operators.return_value = {}
    op = BashOperator(task_id="test", bash_command="exit 0;")
    assert is_operator_disabled(op) is False

    mock_disabled_operators.return_value = {"random_string"}
    assert is_operator_disabled(op) is False

    mock_disabled_operators.return_value = {
        "airflow.providers.standard.operators.bash.BashOperator",
        "airflow.providers.standard.operators.python.PythonOperator",
    }
    assert is_operator_disabled(op) is True


@patch("airflow.providers.openlineage.conf.include_full_task_info")
def test_includes_full_task_info(mock_include_full_task_info):
    mock_include_full_task_info.return_value = True
    # There should be no 'bash_command' in excludes and it's not in includes - so
    # it's a good choice for checking TaskInfo vs TaskInfoComplete
    assert (
        "bash_command"
        in get_airflow_run_facet(
            MagicMock(),
            MagicMock(),
            MagicMock(),
            BashOperator(task_id="bash_op", bash_command="sleep 1"),
            MagicMock(),
        )["airflow"].task
    )


@patch("airflow.providers.openlineage.conf.include_full_task_info")
def test_does_not_include_full_task_info(mock_include_full_task_info):
    mock_include_full_task_info.return_value = False
    # There should be no 'bash_command' in excludes and it's not in includes - so
    # it's a good choice for checking TaskInfo vs TaskInfoComplete
    assert (
        "bash_command"
        not in get_airflow_run_facet(
            MagicMock(),
            MagicMock(),
            MagicMock(),
            BashOperator(task_id="bash_op", bash_command="sleep 1"),
            MagicMock(),
        )["airflow"].task
    )


@pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="This test checks serialization only in 3.0 conditions")
def test_serialize_timetable_complex_with_alias():
    from airflow.providers.common.compat.assets import AssetAlias, AssetAll, AssetAny
    from airflow.timetables.simple import AssetTriggeredTimetable

    asset = AssetAny(
        Asset(name="2", uri="test://2", group="test-group"),
        AssetAlias(name="example-alias", group="test-group"),
        Asset(name="3", uri="test://3", group="test-group"),
        AssetAll(AssetAlias("another"), Asset("4")),
    )
    dag = MagicMock()
    dag.timetable = AssetTriggeredTimetable(asset)
    dag_info = DagInfo(dag)

    assert dag_info.timetable == {
        "asset_condition": {
            "__type": DagAttributeTypes.ASSET_ANY,
            "objects": [
                {
                    "__type": DagAttributeTypes.ASSET,
                    "extra": {},
                    "uri": "test://2/",
                    "name": "2",
                    "group": "test-group",
                },
                {
                    "__type": DagAttributeTypes.ASSET_ALIAS,
                    "name": "example-alias",
                    "group": "test-group",
                },
                {
                    "__type": DagAttributeTypes.ASSET,
                    "extra": {},
                    "uri": "test://3/",
                    "name": "3",
                    "group": "test-group",
                },
                {
                    "__type": DagAttributeTypes.ASSET_ALL,
                    "objects": [
                        {
                            "__type": DagAttributeTypes.ASSET_ALIAS,
                            "name": "another",
                            "group": "asset",
                        },
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "extra": {},
                            "uri": "4",
                            "name": "4",
                            "group": "asset",
                        },
                    ],
                },
            ],
        }
    }


@pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="This test checks serialization only in 3.0 conditions")
def test_serialize_timetable_single_asset():
    dag = DAG(dag_id="test", start_date=datetime.datetime(2025, 1, 1), schedule=Asset("a"))
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "asset_condition": {
            "__type": DagAttributeTypes.ASSET,
            "uri": "a",
            "name": "a",
            "group": "asset",
            "extra": {},
        }
    }


@pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="This test checks serialization only in 3.0 conditions")
def test_serialize_timetable_list_of_assets():
    dag = DAG(dag_id="test", start_date=datetime.datetime(2025, 1, 1), schedule=[Asset("a"), Asset("b")])
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "asset_condition": {
            "__type": DagAttributeTypes.ASSET_ALL,
            "objects": [
                {"__type": DagAttributeTypes.ASSET, "uri": "a", "name": "a", "group": "asset", "extra": {}},
                {"__type": DagAttributeTypes.ASSET, "uri": "b", "name": "b", "group": "asset", "extra": {}},
            ],
        }
    }


@pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="This test checks serialization only in 3.0 conditions")
def test_serialize_timetable_with_complex_logical_condition():
    dag = DAG(
        dag_id="test",
        start_date=datetime.datetime(2025, 1, 1),
        schedule=(Asset("ds1", extra={"some_extra": 1}) | Asset("ds2"))
        & (Asset("ds3") | Asset("ds4", extra={"another_extra": 345})),
    )
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "asset_condition": {
            "__type": DagAttributeTypes.ASSET_ALL,
            "objects": [
                {
                    "__type": DagAttributeTypes.ASSET_ANY,
                    "objects": [
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds1",
                            "extra": {"some_extra": 1},
                            "name": "ds1",
                            "group": "asset",
                        },
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds2",
                            "extra": {},
                            "name": "ds2",
                            "group": "asset",
                        },
                    ],
                },
                {
                    "__type": DagAttributeTypes.ASSET_ANY,
                    "objects": [
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds3",
                            "extra": {},
                            "name": "ds3",
                            "group": "asset",
                        },
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds4",
                            "extra": {"another_extra": 345},
                            "name": "ds4",
                            "group": "asset",
                        },
                    ],
                },
            ],
        }
    }


@pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="This test checks serialization only in 3.0 conditions")
def test_serialize_timetable_with_dataset_or_time_schedule():
    from airflow.timetables.assets import AssetOrTimeSchedule
    from airflow.timetables.trigger import CronTriggerTimetable

    dag = DAG(
        dag_id="test",
        start_date=datetime.datetime(2025, 1, 1),
        schedule=AssetOrTimeSchedule(
            timetable=CronTriggerTimetable("0 0 * 3 *", timezone="UTC"),
            assets=(Asset("ds1", extra={"some_extra": 1}) | Asset("ds2"))
            & (Asset("ds3") | Asset("ds4", extra={"another_extra": 345})),
        ),
    )
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "timetable": {
            Encoding.TYPE: "airflow.timetables.trigger.CronTriggerTimetable",
            Encoding.VAR: {
                "expression": "0 0 * 3 *",
                "timezone": "UTC",
                "interval": 0.0,
                "run_immediately": False,
            },
        },
        "asset_condition": {
            "__type": DagAttributeTypes.ASSET_ALL,
            "objects": [
                {
                    "__type": DagAttributeTypes.ASSET_ANY,
                    "objects": [
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds1",
                            "extra": {"some_extra": 1},
                            "name": "ds1",
                            "group": "asset",
                        },
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds2",
                            "extra": {},
                            "name": "ds2",
                            "group": "asset",
                        },
                    ],
                },
                {
                    "__type": DagAttributeTypes.ASSET_ANY,
                    "objects": [
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds3",
                            "extra": {},
                            "name": "ds3",
                            "group": "asset",
                        },
                        {
                            "__type": DagAttributeTypes.ASSET,
                            "uri": "ds4",
                            "extra": {"another_extra": 345},
                            "name": "ds4",
                            "group": "asset",
                        },
                    ],
                },
            ],
        },
    }


@pytest.mark.skipif(
    AIRFLOW_V_3_0_PLUS,
    reason="This test checks serialization only in 2.10 conditions",
)
def test_serialize_timetable_2_10_complex_with_alias():
    from airflow.providers.common.compat.assets import AssetAlias, AssetAll, AssetAny
    from airflow.timetables.simple import DatasetTriggeredTimetable

    asset = AssetAny(
        Asset("2"),
        AssetAlias("example-alias"),
        Asset("3"),
        AssetAll(AssetAlias("this-should-not-be-seen"), Asset("4")),
    )

    dag = MagicMock()
    dag.timetable = DatasetTriggeredTimetable(asset)
    dag_info = DagInfo(dag)

    assert dag_info.timetable == {
        "dataset_condition": {
            "__type": DagAttributeTypes.DATASET_ANY,
            "objects": [
                {"__type": DagAttributeTypes.DATASET, "extra": None, "uri": "2"},
                {"__type": DagAttributeTypes.DATASET_ANY, "objects": []},
                {"__type": DagAttributeTypes.DATASET, "extra": None, "uri": "3"},
                {
                    "__type": DagAttributeTypes.DATASET_ALL,
                    "objects": [
                        {"__type": DagAttributeTypes.DATASET_ANY, "objects": []},
                        {"__type": DagAttributeTypes.DATASET, "extra": None, "uri": "4"},
                    ],
                },
            ],
        }
    }


@pytest.mark.skipif(
    AIRFLOW_V_3_0_PLUS,
    reason="This test checks serialization only in 2.10 conditions",
)
def test_serialize_timetable_2_10_single_asset():
    dag = DAG(dag_id="test", start_date=datetime.datetime(2025, 1, 1), schedule=Asset("a"))
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "dataset_condition": {"__type": DagAttributeTypes.DATASET, "uri": "a", "extra": None}
    }


@pytest.mark.skipif(
    AIRFLOW_V_3_0_PLUS,
    reason="This test checks serialization only in 2.10 conditions",
)
def test_serialize_timetable_2_10_list_of_assets():
    dag = DAG(dag_id="test", start_date=datetime.datetime(2025, 1, 1), schedule=[Asset("a"), Asset("b")])
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "dataset_condition": {
            "__type": DagAttributeTypes.DATASET_ALL,
            "objects": [
                {"__type": DagAttributeTypes.DATASET, "extra": None, "uri": "a"},
                {"__type": DagAttributeTypes.DATASET, "extra": None, "uri": "b"},
            ],
        }
    }


@pytest.mark.skipif(
    AIRFLOW_V_3_0_PLUS,
    reason="This test checks serialization only in 2.10 conditions",
)
def test_serialize_timetable_2_10_with_complex_logical_condition():
    dag = DAG(
        dag_id="test",
        start_date=datetime.datetime(2025, 1, 1),
        schedule=(Asset("ds1", extra={"some_extra": 1}) | Asset("ds2"))
        & (Asset("ds3") | Asset("ds4", extra={"another_extra": 345})),
    )
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "dataset_condition": {
            "__type": DagAttributeTypes.DATASET_ALL,
            "objects": [
                {
                    "__type": DagAttributeTypes.DATASET_ANY,
                    "objects": [
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds1", "extra": {"some_extra": 1}},
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds2", "extra": None},
                    ],
                },
                {
                    "__type": DagAttributeTypes.DATASET_ANY,
                    "objects": [
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds3", "extra": None},
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds4", "extra": {"another_extra": 345}},
                    ],
                },
            ],
        }
    }


@pytest.mark.skipif(
    AIRFLOW_V_3_0_PLUS,
    reason="This test checks serialization only in 2.10 conditions",
)
def test_serialize_timetable_2_10_with_dataset_or_time_schedule():
    from airflow.timetables.datasets import DatasetOrTimeSchedule
    from airflow.timetables.trigger import CronTriggerTimetable

    dag = DAG(
        dag_id="test",
        start_date=datetime.datetime(2025, 1, 1),
        schedule=DatasetOrTimeSchedule(
            timetable=CronTriggerTimetable("0 0 * 3 *", timezone="UTC"),
            datasets=(Asset("ds1", extra={"some_extra": 1}) | Asset("ds2"))
            & (Asset("ds3") | Asset("ds4", extra={"another_extra": 345})),
        ),
    )
    dag_info = DagInfo(dag)
    assert dag_info.timetable == {
        "timetable": {
            "__type": "airflow.timetables.trigger.CronTriggerTimetable",
            "__var": {"expression": "0 0 * 3 *", "timezone": "UTC", "interval": 0.0},
        },
        "dataset_condition": {
            "__type": DagAttributeTypes.DATASET_ALL,
            "objects": [
                {
                    "__type": DagAttributeTypes.DATASET_ANY,
                    "objects": [
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds1", "extra": {"some_extra": 1}},
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds2", "extra": None},
                    ],
                },
                {
                    "__type": DagAttributeTypes.DATASET_ANY,
                    "objects": [
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds3", "extra": None},
                        {"__type": DagAttributeTypes.DATASET, "uri": "ds4", "extra": {"another_extra": 345}},
                    ],
                },
            ],
        },
    }


@pytest.mark.parametrize(
    ("airflow_version", "ol_version"),
    [
        ("2.9.3", "1.12.2"),
        ("2.10.1", "1.13.0"),
        ("3.0.0", "1.14.0"),
    ],
)
def test_get_processing_engine_facet(airflow_version, ol_version):
    with patch("airflow.providers.openlineage.utils.utils.AIRFLOW_VERSION", airflow_version):
        with patch("airflow.providers.openlineage.utils.utils.OPENLINEAGE_PROVIDER_VERSION", ol_version):
            result = get_processing_engine_facet()
            assert result["processing_engine"].version == airflow_version
            assert result["processing_engine"].openlineageAdapterVersion == ol_version



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import TYPE_CHECKING

from airflow.providers.openlineage import conf
from airflow.providers.openlineage.plugins.adapter import OpenLineageAdapter
from airflow.providers.openlineage.utils.utils import (
    get_job_name,
    get_parent_information_from_dagrun_conf,
    get_root_information_from_dagrun_conf,
)
from airflow.providers.openlineage.version_compat import AIRFLOW_V_3_0_PLUS

if TYPE_CHECKING:
    from airflow.providers.common.compat.sdk import TaskInstance


def lineage_job_namespace():
    """
    Macro function which returns Airflow OpenLineage namespace.

    .. seealso::
        For more information take a look at the guide:
        :ref:`howto/macros:openlineage`
    """
    return conf.namespace()


def lineage_job_name(task_instance: TaskInstance):
    """
    Macro function which returns Airflow task name in OpenLineage format (`<dag_id>.<task_id>`).

    .. seealso::
        For more information take a look at the guide:
        :ref:`howto/macros:openlineage`
    """
    return get_job_name(task_instance)


def lineage_run_id(task_instance: TaskInstance):
    """
    Macro function which returns the generated run id (UUID) for a given task.

    This can be used to forward the run id from a task to a child run so the job hierarchy is preserved.

    .. seealso::
        For more information take a look at the guide:
        :ref:`howto/macros:openlineage`
    """
    return OpenLineageAdapter.build_task_instance_run_id(
        dag_id=task_instance.dag_id,
        task_id=task_instance.task_id,
        try_number=task_instance.try_number,
        logical_date=_get_logical_date(task_instance),
        map_index=task_instance.map_index,
    )


def lineage_parent_id(task_instance: TaskInstance):
    """
    Macro function which returns a unique identifier of given task that can be used to create ParentRunFacet.

    This identifier is composed of the namespace, job name, and generated run id for given task, structured
    as '{namespace}/{job_name}/{run_id}'. This can be used to forward task information from a task to a child
    run so the job hierarchy is preserved. Child run can easily create ParentRunFacet from these information.

    .. seealso::
        For more information take a look at the guide:
        :ref:`howto/macros:openlineage`
    """
    return "/".join(
        (
            lineage_job_namespace(),
            lineage_job_name(task_instance),
            lineage_run_id(task_instance),
        )
    )


def lineage_root_parent_id(task_instance: TaskInstance):
    """
    Macro function which returns a unique identifier of given task that can be used to create root information for ParentRunFacet.

    This identifier is composed of the namespace, dag name, and generated run id for given dag, structured
    as '{namespace}/{job_name}/{run_id}'.

    .. seealso::
        For more information take a look at the guide:
        :ref:`howto/macros:openlineage`
    """
    return "/".join(
        (
            lineage_root_job_namespace(task_instance),
            lineage_root_job_name(task_instance),
            lineage_root_run_id(task_instance),
        )
    )


def lineage_root_job_name(task_instance: TaskInstance):
    root_parent_job_name = _get_ol_root_id("root_parent_job_name", task_instance)
    if root_parent_job_name:
        return root_parent_job_name
    return task_instance.dag_id


def lineage_root_run_id(task_instance: TaskInstance):
    root_parent_run_id = _get_ol_root_id("root_parent_run_id", task_instance)
    if root_parent_run_id:
        return root_parent_run_id
    return OpenLineageAdapter.build_dag_run_id(
        dag_id=task_instance.dag_id,
        logical_date=_get_logical_date(task_instance),
        clear_number=_get_dag_run_clear_number(task_instance),
    )


def lineage_root_job_namespace(task_instance: TaskInstance):
    root_parent_job_namespace = _get_ol_root_id("root_parent_job_namespace", task_instance)
    if root_parent_job_namespace:
        return root_parent_job_namespace
    return conf.namespace()


def _get_ol_root_id(id_key: str, task_instance: TaskInstance) -> str | None:
    dr_conf = _get_dag_run_conf(task_instance=task_instance)
    # Check DagRun conf for root info
    ol_root_info = get_root_information_from_dagrun_conf(dr_conf=dr_conf)
    if ol_root_info and ol_root_info.get(id_key):
        return ol_root_info[id_key]
    # Then check DagRun conf for parent into that is used as root in case explicit root is missing
    id_key = id_key.replace("root_", "")
    ol_root_info = get_parent_information_from_dagrun_conf(dr_conf=dr_conf)
    if ol_root_info and ol_root_info.get(id_key):
        return ol_root_info[id_key]
    return None


def _get_dagrun_from_ti(task_instance: TaskInstance):
    context = task_instance.get_template_context()
    if getattr(task_instance, "dag_run", None):
        return task_instance.dag_run
    return context["dag_run"]


def _get_dag_run_conf(task_instance: TaskInstance) -> dict:
    dr = _get_dagrun_from_ti(task_instance=task_instance)
    return dr.conf or {}


def _get_dag_run_clear_number(task_instance: TaskInstance):
    dr = _get_dagrun_from_ti(task_instance=task_instance)
    return dr.clear_number


def _get_logical_date(task_instance):
    if AIRFLOW_V_3_0_PLUS:
        dr = _get_dagrun_from_ti(task_instance=task_instance)
        if getattr(dr, "logical_date", None):
            return dr.logical_date
        return dr.run_after
    if getattr(task_instance, "logical_date", None):
        return task_instance.logical_date
    return task_instance.execution_date



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import tempfile
from typing import TYPE_CHECKING, Any
from unittest.mock import MagicMock

import pytest
from openlineage.client.event_v2 import Dataset as OpenLineageDataset
from openlineage.client.facet_v2 import (
    documentation_dataset,
    ownership_dataset,
    schema_dataset,
)

from airflow.models.taskinstance import TaskInstance
from airflow.providers.common.compat.lineage.entities import Column, File, Table, User
from airflow.providers.common.compat.sdk import BaseOperator, Context, ObjectStoragePath
from airflow.providers.openlineage.extractors import OperatorLineage
from airflow.providers.openlineage.extractors.manager import ExtractorManager
from airflow.providers.openlineage.utils.utils import Asset
from airflow.utils.state import State

from tests_common.test_utils.compat import DateTimeSensor, PythonOperator
from tests_common.test_utils.markers import skip_if_force_lowest_dependencies_marker
from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS

if TYPE_CHECKING:
    try:
        from airflow.sdk.api.datamodels._generated import AssetEventDagRunReference, TIRunContext
    except ImportError:
        AssetEventDagRunReference = TIRunContext = Any  # type: ignore[misc, assignment]


@pytest.fixture
def hook_lineage_collector():
    from airflow.lineage import hook
    from airflow.providers.common.compat.lineage.hook import get_hook_lineage_collector

    hlc = hook.HookLineageCollector()
    if AIRFLOW_V_3_0_PLUS:
        from unittest import mock

        with mock.patch(
            "airflow.lineage.hook.get_hook_lineage_collector",
            return_value=hlc,
        ):
            yield get_hook_lineage_collector()
    else:
        hook._hook_lineage_collector = hlc

        yield get_hook_lineage_collector()

        hook._hook_lineage_collector = None


@pytest.mark.parametrize(
    ("uri", "dataset"),
    (
        (
            "s3://bucket1/dir1/file1",
            OpenLineageDataset(namespace="s3://bucket1", name="dir1/file1"),
        ),
        (
            "gs://bucket2/dir2/file2",
            OpenLineageDataset(namespace="gs://bucket2", name="dir2/file2"),
        ),
        (
            "gcs://bucket3/dir3/file3",
            OpenLineageDataset(namespace="gs://bucket3", name="dir3/file3"),
        ),
        (
            "hdfs://namenodehost:8020/file1",
            OpenLineageDataset(namespace="hdfs://namenodehost:8020", name="file1"),
        ),
        (
            "hdfs://namenodehost/file2",
            OpenLineageDataset(namespace="hdfs://namenodehost", name="file2"),
        ),
        (
            "file://localhost/etc/fstab",
            OpenLineageDataset(namespace="file://localhost", name="etc/fstab"),
        ),
        (
            "file:///etc/fstab",
            OpenLineageDataset(namespace="file://", name="etc/fstab"),
        ),
        ("https://test.com", OpenLineageDataset(namespace="https", name="test.com")),
        (
            "https://test.com?param1=test1&param2=test2",
            OpenLineageDataset(namespace="https", name="test.com"),
        ),
        ("file:test.csv", None),
        ("not_an_url", None),
    ),
)
def test_convert_to_ol_dataset_from_object_storage_uri(uri, dataset):
    result = ExtractorManager.convert_to_ol_dataset_from_object_storage_uri(uri)
    assert result == dataset


@pytest.mark.parametrize(
    ("obj", "dataset"),
    (
        (
            OpenLineageDataset(namespace="n1", name="f1"),
            OpenLineageDataset(namespace="n1", name="f1"),
        ),
        (
            File(url="s3://bucket1/dir1/file1"),
            OpenLineageDataset(namespace="s3://bucket1", name="dir1/file1"),
        ),
        (
            File(url="gs://bucket2/dir2/file2"),
            OpenLineageDataset(namespace="gs://bucket2", name="dir2/file2"),
        ),
        (
            File(url="gcs://bucket3/dir3/file3"),
            OpenLineageDataset(namespace="gs://bucket3", name="dir3/file3"),
        ),
        (
            File(url="hdfs://namenodehost:8020/file1"),
            OpenLineageDataset(namespace="hdfs://namenodehost:8020", name="file1"),
        ),
        (
            File(url="hdfs://namenodehost/file2"),
            OpenLineageDataset(namespace="hdfs://namenodehost", name="file2"),
        ),
        (
            File(url="file://localhost/etc/fstab"),
            OpenLineageDataset(namespace="file://localhost", name="etc/fstab"),
        ),
        (
            File(url="file:///etc/fstab"),
            OpenLineageDataset(namespace="file://", name="etc/fstab"),
        ),
        (
            File(url="https://test.com"),
            OpenLineageDataset(namespace="https", name="test.com"),
        ),
        (
            Table(cluster="c1", database="d1", name="t1"),
            OpenLineageDataset(namespace="c1", name="d1.t1"),
        ),
        ("gs://bucket2/dir2/file2", None),
        ("not_an_url", None),
    ),
)
def test_convert_to_ol_dataset(obj, dataset):
    result = ExtractorManager.convert_to_ol_dataset(obj)
    assert result == dataset


def test_convert_to_ol_dataset_from_table_with_columns_and_owners():
    table = Table(
        cluster="c1",
        database="d1",
        name="t1",
        columns=[
            Column(name="col1", description="desc1", data_type="type1"),
            Column(name="col2", description="desc2", data_type="type2"),
        ],
        owners=[
            User(email="mike@company.com", first_name="Mike", last_name="Smith"),
            User(email="theo@company.com", first_name="Theo"),
            User(email="smith@company.com", last_name="Smith"),
            User(email="jane@company.com"),
        ],
        description="test description",
    )
    expected_facets = {
        "schema": schema_dataset.SchemaDatasetFacet(
            fields=[
                schema_dataset.SchemaDatasetFacetFields(
                    name="col1",
                    type="type1",
                    description="desc1",
                ),
                schema_dataset.SchemaDatasetFacetFields(
                    name="col2",
                    type="type2",
                    description="desc2",
                ),
            ]
        ),
        "ownership": ownership_dataset.OwnershipDatasetFacet(
            owners=[
                ownership_dataset.Owner(name="user:Mike Smith <mike@company.com>", type=""),
                ownership_dataset.Owner(name="user:Theo <theo@company.com>", type=""),
                ownership_dataset.Owner(name="user:Smith <smith@company.com>", type=""),
                ownership_dataset.Owner(name="user:<jane@company.com>", type=""),
            ]
        ),
        "documentation": documentation_dataset.DocumentationDatasetFacet(description="test description"),
    }
    result = ExtractorManager.convert_to_ol_dataset_from_table(table)
    assert result.namespace == "c1"
    assert result.name == "d1.t1"
    assert result.facets == expected_facets


def test_convert_to_ol_dataset_table():
    table = Table(
        cluster="c1",
        database="d1",
        name="t1",
        columns=[
            Column(name="col1", description="desc1", data_type="type1"),
            Column(name="col2", description="desc2", data_type="type2"),
        ],
        owners=[
            User(email="mike@company.com", first_name="Mike", last_name="Smith"),
            User(email="theo@company.com", first_name="Theo"),
            User(email="smith@company.com", last_name="Smith"),
            User(email="jane@company.com"),
        ],
    )
    expected_facets = {
        "schema": schema_dataset.SchemaDatasetFacet(
            fields=[
                schema_dataset.SchemaDatasetFacetFields(
                    name="col1",
                    type="type1",
                    description="desc1",
                ),
                schema_dataset.SchemaDatasetFacetFields(
                    name="col2",
                    type="type2",
                    description="desc2",
                ),
            ]
        ),
        "ownership": ownership_dataset.OwnershipDatasetFacet(
            owners=[
                ownership_dataset.Owner(name="user:Mike Smith <mike@company.com>", type=""),
                ownership_dataset.Owner(name="user:Theo <theo@company.com>", type=""),
                ownership_dataset.Owner(name="user:Smith <smith@company.com>", type=""),
                ownership_dataset.Owner(name="user:<jane@company.com>", type=""),
            ]
        ),
    }

    result = ExtractorManager.convert_to_ol_dataset(table)
    assert result.namespace == "c1"
    assert result.name == "d1.t1"
    assert result.facets == expected_facets


@skip_if_force_lowest_dependencies_marker
def test_extractor_manager_uses_hook_level_lineage(hook_lineage_collector):
    dagrun = MagicMock()
    task = MagicMock()
    del task.get_openlineage_facets_on_start
    del task.get_openlineage_facets_on_complete
    ti = MagicMock()

    hook_lineage_collector.add_input_asset(None, uri="s3://bucket/input_key")
    hook_lineage_collector.add_output_asset(None, uri="s3://bucket/output_key")
    extractor_manager = ExtractorManager()
    metadata = extractor_manager.extract_metadata(
        dagrun=dagrun, task=task, task_instance_state=None, task_instance=ti
    )

    assert metadata.inputs == [OpenLineageDataset(namespace="s3://bucket", name="input_key")]
    assert metadata.outputs == [OpenLineageDataset(namespace="s3://bucket", name="output_key")]


def test_extractor_manager_does_not_use_hook_level_lineage_when_operator(
    hook_lineage_collector,
):
    class FakeSupportedOperator(BaseOperator):
        def execute(self, context: Context) -> Any:
            pass

        def get_openlineage_facets_on_start(self):
            return OperatorLineage(
                inputs=[OpenLineageDataset(namespace="s3://bucket", name="proper_input_key")]
            )

    dagrun = MagicMock()
    task = FakeSupportedOperator(task_id="test_task_extractor")
    ti = MagicMock()
    hook_lineage_collector.add_input_asset(None, uri="s3://bucket/input_key")

    extractor_manager = ExtractorManager()
    metadata = extractor_manager.extract_metadata(
        dagrun=dagrun, task=task, task_instance_state=None, task_instance=ti
    )

    # s3://bucket/input_key not here - use data from operator
    assert metadata.inputs == [OpenLineageDataset(namespace="s3://bucket", name="proper_input_key")]
    assert metadata.outputs == []


@pytest.mark.skipif(
    AIRFLOW_V_3_0_PLUS,
    reason="Test for hook level lineage in Airflow < 3.0",
)
def test_extractor_manager_gets_data_from_pythonoperator(session, dag_maker, hook_lineage_collector):
    path = None
    with tempfile.NamedTemporaryFile() as f:
        path = f.name
        with dag_maker():

            def use_read():
                storage_path = ObjectStoragePath(path)
                with storage_path.open("w") as out:
                    out.write("test")

            task = PythonOperator(task_id="test_task_extractor_pythonoperator", python_callable=use_read)

    dr = dag_maker.create_dagrun()
    ti = TaskInstance(task=task, run_id=dr.run_id)
    ti.refresh_from_db()
    ti.state = State.QUEUED
    session.merge(ti)
    session.commit()

    ti.run()

    datasets = hook_lineage_collector.collected_assets

    assert len(datasets.outputs) == 1
    assert datasets.outputs[0].asset == Asset(uri=path)


@pytest.mark.db_test
@pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="Task SDK related test")
def test_extractor_manager_gets_data_from_pythonoperator_tasksdk(session, hook_lineage_collector, run_task):
    path = None
    with tempfile.NamedTemporaryFile() as f:
        path = f.name

        def use_read():
            storage_path = ObjectStoragePath(path)
            with storage_path.open("w") as out:
                out.write("test")

    task = PythonOperator(task_id="test_task_extractor_pythonoperator", python_callable=use_read)

    run_task(task, dag_id="test_hookcollector_dag")

    datasets = hook_lineage_collector.collected_assets

    assert len(datasets.outputs) == 1
    assert datasets.outputs[0].asset == Asset(uri=path)


def test_extract_inlets_and_outlets_with_operator():
    inlets = [OpenLineageDataset(namespace="namespace1", name="name1")]
    outlets = [OpenLineageDataset(namespace="namespace2", name="name2")]

    extractor_manager = ExtractorManager()
    task = PythonOperator(task_id="task_id", python_callable=lambda x: x, inlets=inlets, outlets=outlets)
    lineage = OperatorLineage()
    extractor_manager.extract_inlets_and_outlets(lineage, task)
    assert lineage.inputs == inlets
    assert lineage.outputs == outlets


def test_extract_inlets_and_outlets_with_sensor():
    inlets = [OpenLineageDataset(namespace="namespace1", name="name1")]
    outlets = [OpenLineageDataset(namespace="namespace2", name="name2")]

    extractor_manager = ExtractorManager()
    task = DateTimeSensor(
        task_id="task_id", target_time="2025-04-04T08:48:13.713922+00:00", inlets=inlets, outlets=outlets
    )
    lineage = OperatorLineage()
    extractor_manager.extract_inlets_and_outlets(lineage, task)
    assert lineage.inputs == inlets
    assert lineage.outputs == outlets



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

import inspect
from collections.abc import Callable

from openlineage.client.facet_v2 import source_code_job

from airflow.providers.openlineage import conf
from airflow.providers.openlineage.extractors.base import BaseExtractor, OperatorLineage
from airflow.providers.openlineage.utils.utils import get_unknown_source_attribute_run_facet

"""
:meta private:
"""


class PythonExtractor(BaseExtractor):
    """
    Extract executed source code and put it into SourceCodeJobFacet.

    This extractor provides visibility on what particular task does by extracting
    executed source code and putting it into SourceCodeJobFacet. It does not extract
    datasets yet.

    :meta private:
    """

    @classmethod
    def get_operator_classnames(cls) -> list[str]:
        return ["PythonOperator"]

    def _execute_extraction(self) -> OperatorLineage | None:
        source_code = self.get_source_code(self.operator.python_callable)
        job_facet: dict = {}
        if conf.is_source_enabled() and source_code:
            job_facet = {
                "sourceCode": source_code_job.SourceCodeJobFacet(
                    language="python",
                    # We're on worker and should have access to DAG files
                    sourceCode=source_code,
                )
            }
        else:
            self.log.debug(
                "OpenLineage disable_source_code option is on - no source code is extracted.",
            )

        return OperatorLineage(
            job_facets=job_facet,
            # The PythonOperator is recorded as an "unknownSource" even though we have an extractor,
            # as the <i>data lineage</i> cannot be determined from the operator directly.
            run_facets=get_unknown_source_attribute_run_facet(task=self.operator, name="PythonOperator"),
        )

    def get_source_code(self, callable: Callable) -> str | None:
        try:
            return inspect.getsource(callable)
        except TypeError:
            # Trying to extract source code of builtin_function_or_method
            return str(callable)
        except OSError:
            self.log.warning(
                "Can't get source code facet of PythonOperator %s",
                self.operator.task_id,
            )
        return None

    def extract(self) -> OperatorLineage | None:
        return super().extract()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from copy import deepcopy

from airflow.config_templates.airflow_local_settings import DEFAULT_LOGGING_CONFIG

LOGGING_CONFIG = deepcopy(DEFAULT_LOGGING_CONFIG)

LOGGING_CONFIG["root"]["level"] = "INFO"


def logger(level):
    return {
        "handlers": ["console", "task"],
        "level": level,
        "propagate": True,
    }


levels = {
    "openlineage": "DEBUG",
    "great_expectations": "INFO",
    "airflow.lineage": "DEBUG",
    "botocore": "INFO",
}

for el, lvl in levels.items():
    LOGGING_CONFIG["loggers"][el] = logger(lvl)
